import 'package:flutter/material.dart';
import 'package:simple_animations/simple_animations.dart';

class GenericFadeTransition extends StatelessWidget {
  final WidgetBuilder builder;
  final Curve curve;
  final Duration delay;
  final Duration duration;
  final Offset initialOffset;
  final Offset endOffset;
  final void Function()? onStart;
  final void Function()? onComplete;

  GenericFadeTransition({
    required this.builder,
    this.delay = const Duration(),
    this.duration = const Duration(seconds: 1),
    this.endOffset = const Offset(0, 0),
    this.initialOffset = const Offset(0, 100),
    this.curve = Curves.ease,
    this.onStart,
    this.onComplete,
  });

  @override
  Widget build(BuildContext context) {
    final _tween = TimelineTween<_AniProps>(curve: curve)
      ..addScene(begin: delay, duration: duration).animate(
        _AniProps.y,
        tween: Tween(
          begin: initialOffset.dy,
          end: endOffset.dy,
        ),
      )
      ..addScene(begin: delay, duration: duration).animate(
        _AniProps.x,
        tween: Tween(
          begin: initialOffset.dx,
          end: endOffset.dx,
        ),
      )
      ..addScene(begin: delay, duration: duration).animate(
        _AniProps.opacity,
        tween: Tween(
          begin: 0.0,
          end: 1.0,
        ),
      );

    return PlayAnimation<TimelineValue<_AniProps>>(
      tween: _tween,
      duration: duration,
      curve: curve,
      child: builder(context),
      delay: delay,
      onComplete: onComplete,
      onStart: onStart,
      builder: (context, animationChild, value) {
        return Transform.translate(
          offset: Offset(
            value.get(_AniProps.x),
            value.get(_AniProps.y),
          ),
          child: Opacity(
            opacity: value.get(_AniProps.opacity),
            child: animationChild,
          ),
        );
      },
    );
  }
}

enum _AniProps { x, y, opacity }
